@@ -27,10 +27,12 @@ sub attach_db {
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my $plugin = $sf->{i}{plugin};
+    my $db = $sf->{d}{db};
     my $attached_db;
-    if ( -s $sf->{i}{f_attached_db} ) {
+    if ( -f $sf->{i}{f_attached_db} ) {
         my $h_ref = $ax->read_json( $sf->{i}{f_attached_db} ) // {};
-        $attached_db = $h_ref->{$sf->{d}{db}} // {};
+        $attached_db = $h_ref->{$plugin}{$db} // {};
     }
     my $dbh = $sf->{d}{dbh};
     my $old_idx = 0;
@@ -38,7 +40,7 @@ sub attach_db {
     ATTACH: while ( 1 ) {
         my @tmp_info = ( $sf->{d}{db_string} );
         for my $key ( sort keys %$attached_db ) {
-            push @tmp_info, __attach_stmt( $dbh, $attached_db->{$key}, $key );
+            push @tmp_info, $sf->attach_stmt( $dbh, $attached_db->{$key}, $key );
         }
         push @tmp_info, '';
         my @pre = ( undef );
@@ -62,9 +64,9 @@ sub attach_db {
             }
             $old_idx = $idx;
         }
-        my $db = $menu->[$idx];
+        my $db_to_attach = $menu->[$idx];
         my $tr = Term::Form::ReadLine->new( $sf->{i}{tr_default} );
-        push @tmp_info, 'DB: ' . $db;
+        push @tmp_info, 'DB: ' . $db_to_attach;
         my $bu_attached_db = { %$attached_db };
 
         ALIAS: while ( 1 ) {
@@ -80,9 +82,9 @@ sub attach_db {
                 next ATTACH;
             }
             else {
-                $attached_db->{$alias} = $db;
+                $attached_db->{$alias} = $db_to_attach;
                 my @tmp_info = ( $sf->{d}{db_string} );
-                push @tmp_info, map { __attach_stmt( $dbh, $attached_db->{$_}, $_ ) } sort keys %$attached_db;
+                push @tmp_info, map { $sf->attach_stmt( $dbh, $attached_db->{$_}, $_ ) } sort keys %$attached_db;
                 push @tmp_info, '';
                 my $info = join( "\n", @tmp_info );
                 # Choose
@@ -96,7 +98,7 @@ sub attach_db {
                     next ALIAS;
                 }
                 my $h_ref = $ax->read_json( $sf->{i}{f_attached_db} ) // {};
-                $h_ref->{$sf->{d}{db}} = $attached_db;
+                $h_ref->{$plugin}{$db} = $attached_db;
                 $ax->write_json( $sf->{i}{f_attached_db}, $h_ref );
                 return 1;
             }
@@ -105,10 +107,10 @@ sub attach_db {
 }
 
 
-
-sub __attach_stmt {
-    my ( $dbh, $db, $alias ) = @_;
-    return sprintf "ATTACH DATABASE %s AS %s", $dbh->quote_identifier( $db ), $dbh->quote( $alias );
+sub attach_stmt {
+    my ( $sf, $dbh, $db_to_attach, $alias ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    return sprintf "ATTACH DATABASE %s AS %s", $dbh->quote( $db_to_attach ), $ax->quote_alias( $alias );
 }
 
 
@@ -116,10 +118,14 @@ sub detach_db {
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my $file_fs = $sf->{i}{f_attached_db};
+    my $plugin = $sf->{i}{plugin};
+    my $db = $sf->{d}{db};
+
     my $attached_db;
-    if ( -s $sf->{i}{f_attached_db} ) {
-        my $h_ref = $ax->read_json( $sf->{i}{f_attached_db} ) // {};
-        $attached_db = $h_ref->{$sf->{d}{db}} // {};
+    if ( -f $file_fs ) {
+        my $h_ref = $ax->read_json( $file_fs ) // {};
+        $attached_db = $h_ref->{$plugin}{$db} // {};
     }
     if ( ! %$attached_db ) {
         my $info = $sf->{d}{db_string};
@@ -160,7 +166,7 @@ sub detach_db {
         my $detached_alias = $aliases[$idx - @pre];
         my $detached_db = delete $attached_db->{$detached_alias};
         my $dbh = $sf->{d}{dbh};
-        $prompt = sprintf "DETACH DATABASE %s  (%s)", $dbh->quote( $detached_alias ), $dbh->quote_identifier( $detached_db );
+        $prompt = sprintf "DETACH DATABASE %s  (%s)", $ax->quote_alias( $detached_alias ), $dbh->quote( $detached_db );
         # Choose
         my $confirm = $tc->choose(
             [ undef, 'YES' ],
@@ -171,14 +177,25 @@ sub detach_db {
             $attached_db->{$detached_alias} = $detached_db;
             next DETACH;
         }
-        my $h_ref = $ax->read_json( $sf->{i}{f_attached_db} ) // {};
+        my $h_ref = $ax->read_json( $file_fs ) // {};
         if ( %$attached_db ) {
-            $h_ref->{$sf->{d}{db}} = $attached_db;
+            $h_ref->{$plugin}{$db} = $attached_db;
         }
         else {
-            delete $h_ref->{$sf->{d}{db}};
+            delete $h_ref->{$plugin}{$db};
+            if ( ! %{$h_ref->{$plugin}} ) {
+                delete $h_ref->{$plugin};
+            }
+        }
+        if ( ! %$h_ref ) {
+            if ( -f $file_fs && ! eval { unlink $file_fs or die "unlink '$file_fs': $!" } ) {
+                $ax->print_error_message( $@ );
+                $ax->write_json( $file_fs, $h_ref ) if -f $file_fs;
+            }
+        }
+        else {
+            $ax->write_json( $file_fs, $h_ref );
         }
-        $ax->write_json( $sf->{i}{f_attached_db}, $h_ref );
         return 1;
     }
 }
