@@ -18,7 +18,8 @@ use Term::Form::ReadLine qw();
 
 use App::DBBrowser::Auxil;
 use App::DBBrowser::GetContent;
-use App::DBBrowser::Opt::Set;
+use App::DBBrowser::Options;
+use App::DBBrowser::Options::ReadWrite;
 use App::DBBrowser::From::Subquery;
 use App::DBBrowser::Table::CommitWriteSQL;
 
@@ -197,7 +198,7 @@ sub create_table {
                         my $data_types = {}; # data_types memory
 
                         EDIT_COLUMN_TYPES: while( 1 ) {
-                            $data_types = $sf->__edit_column_types( $sql, $data_types );
+                            $data_types = $sf->__edit_column_types( $sql, $source, $data_types );
                             if ( ! $data_types ) {
                                 $sql->{ct_column_definitions} = [ @unquoted_ct_column_definitions ];
                                 $sql->{insert_col_names} = [ @unquoted_insert_col_names ];
@@ -311,8 +312,10 @@ sub __get_column_names {
         }
         my $header_row;
         if ( $menu->[$idx] eq $hidden ) {
-            my $opt_set = App::DBBrowser::Opt::Set->new( $sf->{i}, $sf->{o} );
-            $opt_set->set_options( 'create' );
+            my $op = App::DBBrowser::Options->new( $sf->{i}, $sf->{o} );
+            my $op_rw = App::DBBrowser::Options::ReadWrite->new( $sf->{i}, $sf->{o} );
+            $op->config_groups( [ { name => 'group_create_table', text => "- Create Table" } ], $sf->{i}{plugin} );
+            $op_rw->read_config_file( $sf->{i}{driver}, $sf->{i}{plugin} );
             next;
         }
         elsif ( $menu->[$idx] eq $first_row ) {
@@ -357,18 +360,18 @@ sub __autoincrement_column {
 
 
 sub __primary_key_autoincrement_constraint {
-    # provide "primary_key_autoincrement_constraint" only if also "first_col_is_autoincrement" is available
+    # provide "primary_key_autoincrement_constraint" only if also "first_col_is_autoincrement" in InsertUpdateDelete.pm is available
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $driver = $sf->{i}{driver};
-    if ( $driver eq 'SQLite' ) {
+    my $dbms = $sf->{i}{dbms};
+    if ( $dbms eq 'SQLite' ) {
         return "INTEGER PRIMARY KEY";
     }
-    if ( $driver =~ /^(?:mysql|MariaDB)\z/ ) {
+    if ( $dbms =~ /^(?:mysql|MariaDB)\z/ ) {
         return "INT NOT NULL AUTO_INCREMENT PRIMARY KEY";
         # mysql: NOT NULL added automatically with AUTO_INCREMENT
     }
-    if ( $driver eq 'Pg' ) {
+    if ( $dbms eq 'Pg' ) {
         my $pg_major_version = $ax->major_server_version();
         if ( $pg_major_version >= 10 ) {
             # since PostgreSQL version 10
@@ -378,7 +381,7 @@ sub __primary_key_autoincrement_constraint {
             return "SERIAL PRIMARY KEY";
         }
     }
-    if ( $driver eq 'Firebird' ) {
+    if ( $dbms eq 'Firebird' ) {
         my $firebird_major_version = $ax->major_server_version();
         if ( $firebird_major_version >= 4 ) {
             return "INT GENERATED ALWAYS AS IDENTITY PRIMARY KEY";
@@ -387,15 +390,18 @@ sub __primary_key_autoincrement_constraint {
             return "INT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY";
         }
     }
-    if ( $driver eq 'DB2' ) {
+    if ( $dbms eq 'DB2' ) {
         return "INT NOT NULL GENERATED ALWAYS AS IDENTITY PRIMARY KEY";
     }
-    if ( $driver eq 'Oracle' ) {
+    if ( $dbms eq 'Oracle' ) {
         my $ora_major_server_version = $ax->major_server_version();
         if ( $ora_major_server_version >= 12 ) {
             return "NUMBER GENERATED ALWAYS AS IDENTITY";   # Oracle 12c or greater
         }
     }
+    if ( $dbms eq 'MSSQL' ) {
+        return "INT IDENTITY(1,1) PRIMARY KEY";
+    }
 }
 
 
@@ -428,22 +434,32 @@ sub __edit_column_names {
 
 
 sub __edit_column_types {
-    my ( $sf, $sql, $data_types ) = @_;
+    my ( $sf, $sql, $source, $data_types ) = @_;
     my $tf = Term::Form->new( $sf->{i}{tf_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $dbms = $sf->{i}{dbms};
     my $fields;
     if ( ! %$data_types && $sf->{o}{create}{data_type_guessing} ) {
         $ax->print_sql_info( $ax->get_sql_info( $sql ), 'Column data types: guessing ... ' );
-        require SQL::Type::Guess;
-        my $g = SQL::Type::Guess->new();
         my $header = $sql->{insert_col_names}; #
         my $table  = $sql->{insert_args};
-        my @aoh;
-        for my $row ( @$table ) {
-            push @aoh, {
-                map { $header->[$_] => $row->[$_] } 0 .. $#$header
-            };
+        my $encoding;
+        if ( $sf->{o}{create}{encode_for_data_type_guessing} ) { # MSSQL DB2 Oracle
+            $encoding = $source->{source_type} eq 'plain' ? 'locale' : $sf->{o}{insert}{file_encoding};
         }
+        my @aoh;
+        if ( length $encoding ) {
+            for my $row ( @$table ) {
+                push @aoh, { map { $header->[$_] => Encode::encode( $encoding, $row->[$_] ) } 0 .. $#$header };
+            }
+        }
+        else {
+            for my $row ( @$table ) {
+                push @aoh, { map { $header->[$_] => $row->[$_] } 0 .. $#$header };
+            }
+        }
+        require SQL::Type::Guess;
+        my $g = SQL::Type::Guess->new();
         $g->guess( @aoh );
         my $tmp = $g->column_type;
         $data_types = { map { $_ => uc( $tmp->{$_} ) } keys %$tmp };
@@ -459,12 +475,12 @@ sub __edit_column_types {
         unshift @$fields, [ $sql->{ct_column_definitions}[0], $sf->__primary_key_autoincrement_constraint() ];
         $read_only = [ 0 ];
     }
-    if ( $sf->{i}{driver} =~ /^(?:Pg|Firebird|Informix|Oracle)\z/ ) {
+    if ( $dbms =~ /^(?:Pg|DuckDB|Firebird|Informix|Oracle)\z/ ) {
         for my $field ( @$fields ) {
             if ( defined $field->[1] && $field->[1] eq 'DATETIME' ) {
-                $field->[1] = 'TIMESTAMP'                 if $sf->{i}{driver} =~ /^(?:Pg|Firebird)\z/;
-                $field->[1] = 'DATE'                      if $sf->{i}{driver} eq 'Oracle';
-                $field->[1] = 'DATETIME YEAR TO FRACTION' if $sf->{i}{driver} eq 'Informix';
+                $field->[1] = 'TIMESTAMP'                 if $dbms =~ /^(?:Pg|DuckDB|Firebird)\z/;
+                $field->[1] = 'DATE'                      if $dbms eq 'Oracle';
+                $field->[1] = 'DATETIME YEAR TO FRACTION' if $dbms eq 'Informix';
                 # Informix: DATETIME largest_qualifier TO smallest_qualifier
             }
         }
